# -*- coding: utf-8 -*-
##  Photini - a simple photo metadata editor.
##  http://github.com/jim-easterbrook/Photini
##  Copyright (C) 2012-18  Jim Easterbrook  jim@jim-easterbrook.me.uk
##
##  This program is free software: you can redistribute it and/or
##  modify it under the terms of the GNU General Public License as
##  published by the Free Software Foundation, either version 3 of the
##  License, or (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program.  If not, see
##  <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

import locale
import logging
import os
import webbrowser

import requests
import six

from photini.photinimap import PhotiniMap
from photini.pyqt import (
    Busy, QtCore, QtGui, QtWebEngineWidgets, QtWebKit, QtWidgets)

logger = logging.getLogger(__name__)

class BingMap(PhotiniMap):
    def __init__(self, *arg, **kw):
        super(BingMap, self).__init__(*arg, **kw)
        if not QtWebEngineWidgets:
            self.map.settings().setAttribute(
                QtWebKit.QWebSettings.LocalContentCanAccessRemoteUrls, True)
            self.map.settings().setAttribute(
                QtWebKit.QWebSettings.LocalContentCanAccessFileUrls, True)

    def get_page_elements(self):
        url = 'http://www.bing.com/api/maps/mapcontrol?callback=initialize'
        lang, encoding = locale.getdefaultlocale()
        if lang:
            url += '&mkt={0},ngt'.format(lang.replace('_', '-'))
        else:
            url += '&mkt=ngt'
        if self.app.test_mode:
            url += '&branch=experimental'
        return {
            'head': '''
    <script type="text/javascript">
      var api_key = "{}";
    </script>
    <script type="text/javascript"
      src="{}" async defer>
    </script>
'''.format(self.api_key, url),
            'body': '',
            }

    def show_terms(self):
        # return widget to display map terms and conditions
        layout = QtWidgets.QVBoxLayout()
        widget = QtWidgets.QPushButton(self.tr('Terms of Use'))
        widget.clicked.connect(self.load_tou)
        widget.setStyleSheet('QPushButton { font-size: 10px }')
        layout.addWidget(widget)
        return layout

    @QtCore.pyqtSlot()
    def load_tou(self):
        webbrowser.open_new(
            'http://www.microsoft.com/maps/assets/docs/terms.aspx')

    def do_geocode(self, query='', params={}):
        self.disable_search()
        params['key'] = self.map_status['session_id']
        url = 'http://dev.virtualearth.net/REST/v1/Locations'
        if query:
            url += '/' + query
        with Busy():
            try:
                rsp = requests.get(url, params=params, timeout=5)
            except Exception as ex:
                logger.error(str(ex))
                return []
        if rsp.status_code >= 400:
            logger.error('Search error %d', rsp.status_code)
            return []
        if rsp.headers['X-MS-BM-WS-INFO'] == '1':
            logger.error('Server overload')
        else:
            # re-enable search immediately rather than after timeout
            self.enable_search()
        rsp = rsp.json()
        if rsp['statusCode'] != 200:
            logger.error('Search error %d: %s',
                         rsp['statusCode'], rsp['statusDescription'])
            return []
        resource_sets = rsp['resourceSets']
        if not resource_sets:
            logger.error('No results found')
            return []
        return resource_sets

    address_map = {
        'country_code'  : ('countryRegionIso2',),
        'country_name'  : ('countryRegion',),
        'province_state': ('adminDistrict2', 'adminDistrict'),
        'city'          : ('neighborhood', 'locality'),
        'sublocation'   : ('landmark', 'addressLine'),
        }

    def reverse_geocode(self, coords):
        query = coords
        params = {
            'inclnb': '1',
            'incl'  : 'ciso2',
            }
        resource_sets = self.do_geocode(query=query, params=params)
        if not resource_sets:
            return None
        address = resource_sets[0]['resources'][0]['address']
        for key in ('formattedAddress', 'postalCode'):
            if key in address:
                del address[key]
        return address

    def geocode(self, search_string, north, east, south, west):
        params = {
            'q'     : search_string,
            'maxRes': 20,
            'umv'   : '{!r},{!r},{!r},{!r}'.format(south, west, north, east),
            }
        for resource_set in self.do_geocode(params=params):
            for resource in resource_set['resources']:
                south, west, north, east = resource['bbox']
                yield north, east, south, west, resource['name']



# -*- coding: utf-8 -*-
##  Photini - a simple photo metadata editor.
##  http://github.com/jim-easterbrook/Photini
##  Copyright (C) 2012-18  Jim Easterbrook  jim@jim-easterbrook.me.uk
##
##  This program is free software: you can redistribute it and/or
##  modify it under the terms of the GNU General Public License as
##  published by the Free Software Foundation, either version 3 of the
##  License, or (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program.  If not, see
##  <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

import locale
import logging
import os
import re
import webbrowser

import requests

from photini.photinimap import PhotiniMap
from photini.pyqt import Busy, QtCore, QtWidgets

logger = logging.getLogger(__name__)

class GoogleMap(PhotiniMap):
    def get_page_elements(self):
        url = 'http://maps.googleapis.com/maps/api/js?callback=initialize&v=3'
        if self.app.test_mode:
            url += '.exp'
        url += '&key=' + self.api_key
        lang, encoding = locale.getdefaultlocale()
        if lang:
            match = re.match('[a-zA-Z]+[-_]([A-Z]+)', lang)
            if match:
                name = match.group(1)
                if name:
                    url += '&region=' + name
        return {
            'head': '',
            'body': '''
    <script type="text/javascript"
      src="{}" async defer>
    </script>
'''.format(url),
            }

    def show_terms(self):
        # return widget to display map terms and conditions
        layout = QtWidgets.QVBoxLayout()
        widget = QtWidgets.QLabel(self.tr('Search powered by Google'))
        widget.setStyleSheet('QLabel { font-size: 10px }')
        layout.addWidget(widget)
        widget = QtWidgets.QPushButton(self.tr('Terms of Use'))
        widget.clicked.connect(self.load_tou)
        widget.setStyleSheet('QPushButton { font-size: 10px }')
        layout.addWidget(widget)
        return layout

    @QtCore.pyqtSlot()
    def load_tou(self):
        webbrowser.open_new('http://www.google.com/help/terms_maps.html')

    def do_geocode(self, params):
        self.disable_search()
        params['key'] = self.api_key
        lang, encoding = locale.getdefaultlocale()
        if lang:
            params['language'] = lang
        url = 'https://maps.googleapis.com/maps/api/geocode/json'
        with Busy():
            try:
                rsp = requests.get(url, params=params, timeout=5)
            except Exception as ex:
                logger.error(str(ex))
                return []
        if rsp.status_code >= 400:
            logger.error('Search error %d', rsp.status_code)
            return []
        self.enable_search()
        rsp = rsp.json()
        if rsp['status'] != 'OK':
            if 'error_message' in rsp:
                logger.error(
                    'Search error: %s: %s', rsp['status'], rsp['error_message'])
            else:
                logger.error('Search error: %s', rsp['status'])
            return []
        results = rsp['results']
        if not results:
            logger.error('No results found')
            return []
        return results

    address_map = {
        'country_code'  : ('country_code',),
        'country_name'  : ('country',),
        'province_state': ('administrative_area_level_3',
                           'administrative_area_level_2',
                           'administrative_area_level_1'),
        'city'          : ('sublocality_level_2', 'sublocality_level_1',
                           'sublocality', 'neighborhood', 'locality',
                           'postal_town'),
        'sublocation'   : ('establishment', 'point_of_interest', 'premise',
                           'street_number', 'route'),
        }

    def reverse_geocode(self, coords):
        results = self.do_geocode({'latlng': coords})
        if not results:
            return None
        # the first result is the most specific
        address_components = results[0]['address_components']
        # merge in a street address if it's not the first result
        for result in results[1:]:
            if 'street_address' in result['types']:
                address_components += result['address_components']
        address = {}
        for item in address_components:
            type_name = ''
            for name in item['types']:
                if name == 'political':
                    continue
                if len(name) > len(type_name):
                    type_name = name
            if not type_name:
                type_name = 'unknown'
            if type_name in ('postal_code', 'postal_code_suffix'):
                continue
            address[type_name] = item['long_name']
            if type_name == 'country':
                address['country_code'] = item['short_name']
        return address

    def geocode(self, search_string, north, east, south, west):
        params = {
            'address': search_string,
            'bounds' : '{!r},{!r}|{!r},{!r}'.format(south, west, north, east),
            }
        for result in self.do_geocode(params):
            bounds = result['geometry']['viewport']
            yield (bounds['northeast']['lat'], bounds['northeast']['lng'],
                   bounds['southwest']['lat'], bounds['southwest']['lng'],
                   result['formatted_address'])



# -*- coding: utf-8 -*-
##  Photini - a simple photo metadata editor.
##  http://github.com/jim-easterbrook/Photini
##  Copyright (C) 2012-18  Jim Easterbrook  jim@jim-easterbrook.me.uk
##
##  This program is free software: you can redistribute it and/or
##  modify it under the terms of the GNU General Public License as
##  published by the Free Software Foundation, either version 3 of the
##  License, or (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program.  If not, see
##  <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

import locale
import logging
import os
import webbrowser

import requests
import six

from photini.photinimap import PhotiniMap
from photini.pyqt import Busy, Qt, QtCore, QtWidgets, qt_version_info

logger = logging.getLogger(__name__)

class OpenStreetMap(PhotiniMap):
    def get_page_elements(self):
        return {
            'head': '''
    <link rel="stylesheet"
      href="https://unpkg.com/leaflet@1.3.1/dist/leaflet.css" />
    <script type="text/javascript">
      var L_NO_TOUCH = true;
    </script>
    <script type="text/javascript"
      src="https://unpkg.com/leaflet@1.3.1/dist/leaflet.js">
    </script>
''',
            'body': '''
    <script type="text/javascript">
      initialize();
    </script>
''',
            }

    def show_terms(self):
        # return widget to display map terms and conditions
        layout = QtWidgets.QGridLayout()
        widget = QtWidgets.QPushButton(self.tr('Search powered by OpenCage'))
        widget.clicked.connect(self.load_tou_opencage)
        widget.setStyleSheet('QPushButton { font-size: 10px }')
        layout.addWidget(widget, 0, 0)
        widget = QtWidgets.QPushButton(self.tr('Map powered by Leaflet'))
        widget.clicked.connect(self.load_tou_leaflet)
        widget.setStyleSheet('QPushButton { font-size: 10px }')
        layout.addWidget(widget, 0, 1)
        widget = QtWidgets.QPushButton(
            self.tr('Map data ©OpenStreetMap\ncontributors, licensed under ODbL'))
        widget.clicked.connect(self.load_tou_osm)
        widget.setStyleSheet('QPushButton { font-size: 10px }')
        layout.addWidget(widget, 1, 0)
        widget = QtWidgets.QPushButton(
            self.tr('Map tiles by CARTO\nlicensed under CC BY 3.0'))
        widget.clicked.connect(self.load_tou_tiles)
        widget.setStyleSheet('QPushButton { font-size: 10px }')
        layout.addWidget(widget, 1, 1)
        return layout

    @QtCore.pyqtSlot()
    def load_tou_opencage(self):
        webbrowser.open_new('https://geocoder.opencagedata.com/')

    @QtCore.pyqtSlot()
    def load_tou_leaflet(self):
        webbrowser.open_new('http://leafletjs.com/')

    @QtCore.pyqtSlot()
    def load_tou_osm(self):
        webbrowser.open_new('http://www.openstreetmap.org/copyright')

    @QtCore.pyqtSlot()
    def load_tou_tiles(self):
        webbrowser.open_new('https://carto.com/attribution')

    def do_geocode(self, params):
        self.disable_search()
        params['key'] = self.api_key
        params['abbrv'] = '1'
        params['no_annotations'] = '1'
        lang, encoding = locale.getdefaultlocale()
        if lang:
            params['language'] = lang
        with Busy():
            try:
                rsp = requests.get(
                    'https://api.opencagedata.com/geocode/v1/json',
                    params=params, timeout=5)
            except Exception as ex:
                logger.error(str(ex))
                return []
        if rsp.status_code >= 400:
            logger.error('Search error %d', rsp.status_code)
            return []
        rsp = rsp.json()
        status = rsp['status']
        if status['code'] != 200:
            logger.error(
                'Search error %d: %s', status['code'], status['message'])
            return []
        if rsp['total_results'] < 1:
            logger.error('No results found')
            return []
        rate = rsp['rate']
        self.block_timer.setInterval(
            5000 * rate['limit'] // max(rate['remaining'], 1))
        return rsp['results']

    address_map = {
        'world_region'  :('continent',),
        'country_code'  :('country_code', 'ISO_3166-1_alpha-2'),
        'country_name'  :('country',),
        'province_state':('region', 'county', 'state_district', 'state'),
        'city'          :('hamlet', 'locality', 'neighbourhood', 'village',
                          'suburb', 'town', 'city_district', 'city'),
        'sublocation'   :('building', 'house_number',
                          'footway', 'pedestrian', 'road'),
        }

    def reverse_geocode(self, coords):
        results = self.do_geocode({'q': coords})
        if not results:
            return None
        address = results[0]['components']
        for key in ('postcode', 'road_reference', 'road_reference_intl',
                    'state_code', '_type'):
            if key in address:
                del address[key]
        if 'country_code' in address:
            address['country_code'] = address['country_code'].upper()
        return address

    def geocode(self, search_string, north, east, south, west):
        w = east - west
        h = north - south
        if min(w, h) < 10.0:
            lat, lon = self.map_status['centre']
            north = min(lat + 5.0,  90.0)
            south = max(lat - 5.0, -90.0)
            east = lon + 5.0
            west = lon - 5.0
        params = {
            'q'     : search_string,
            'limit' : '20',
            'bounds': '{!r},{!r},{!r},{!r}'.format(west, south, east, north),
            }
        for result in self.do_geocode(params):
            yield (result['bounds']['northeast']['lat'],
                   result['bounds']['northeast']['lng'],
                   result['bounds']['southwest']['lat'],
                   result['bounds']['southwest']['lng'],
                   result['formatted'])
